﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

Static StrConstant k_fullpackageFolder = "root:Packages:imgT"
Static StrConstant k_pvfolder = "imgT_processvariables"
Static StrConstant k_fullpvfolder = "root:Packages:imgT:imgT_processvariables"
Static StrConstant k_fullpanel = "imgT_display#imgT_panel"
Static StrConstant k_imgDisplay = "imgT_display"
Static Constant cbheight = 50

// uncomment to debug
//#define DEBUG

#ifndef DEBUG
#pragma hide=1
#endif

Menu "TracePopup"
	"Remove This Point", /Q, T1_removeBckgrndPoint()
end

// ***
// STRUCTURES

// background settings
Static Structure S_BackgroundParams
	variable plorder
	variable onplane
	variable negatives
endStructure

Function Sf_GetBackgroundParams(sbg)
	STRUCT S_BackgroundParams &sbg
	
	ControlInfo/W=$k_fullpanel setvarPlOrder_tab1
	sbg.plorder = v_value
	ControlInfo/W=$k_fullpanel popupBckonStack_tab1
	sbg.onplane = v_value
	ControlInfo/W=$k_fullpanel popupBckNegs_tab1
	sbg.negatives = v_value
	
	return 0
end

// structure to store calibration variables
Structure S_CalibrateVariables
	// how: 1 pixel, 2 width, 3 height
	variable xyhow
	// panel inputs
	variable pivalue
	variable pilog
	// meters per pixel
	variable mpp
	// image xy size (meters)
	variable xyiwidth
	variable xyiheight
	// store as global
	variable asglobal
	// intensity
	variable ihow
	variable ionstack
	string ieqn
	variable bonstack
EndStructure

Function Sf_GetCalibrationInputs(scv)
	STRUCT S_CalibrateVariables &scv

	ControlInfo/W=$k_fullpanel popupmethodxy_tab1
	scv.xyhow = v_value
	ControlInfo/W=$k_fullpanel psizem_tab1
	scv.pivalue = v_value
	ControlInfo/W=$k_fullpanel psizelm_tab1
	scv.pilog = v_value
	ControlInfo/W=$k_fullpanel checkxyglobal_tab1
	scv.asglobal = v_value
	ControlInfo/W=$k_fullpanel popupmethodcalI_tab1
	scv.ihow = v_value
	ControlInfo/W=$k_fullpanel popupIOnStack_tab1
	scv.ionstack = v_value
	ControlInfo/W=$k_fullpanel setvarcalIEQN_tab1
	scv.ieqn = s_value
	ControlInfo/W=$k_fullpanel popupBckOnStack_tab1
	scv.bonstack = v_value

	return 0
end

Function Sf_SaveCalibrateVariables(scv)
	STRUCT S_CalibrateVariables &scv

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string iwf	
	
	DFREF cwd = GetDataFolderDFR()

	// always save local
	// (clumsy but future proof)
	iwf = "root:" + cwdfolder	
	SetDataFolder $iwf	
	NewDataFolder/O/S $k_pvfolder
	variable/G xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog, ihow, ionstack, bonstack
	string/G ieqn
	SetDataFolder cwd
	iwf += ":" + k_pvfolder
	NVAR/SDFR=$iwf xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog, ihow, ionstack, bonstack
	SVAR/SDFR=$iwf ieqn
	xyhow = scv.xyhow
	mpp = scv.mpp
	xyiwidth = scv.xyiwidth
	xyiheight = scv.xyiheight
	pivalue = scv.pivalue
	pilog = scv.pilog
	ihow = scv.ihow
	ionstack = scv.ionstack
	ieqn = scv.ieqn
	bonstack = scv.bonstack
	
	if (scv.asglobal)
		// validate/create globals anew each time
		// (clumsy but future proof)
		SetDataFolder pf
		NewDataFolder/O/S $k_pvfolder
		variable/G xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog, ihow, ionstack, bonstack
		SetDataFolder cwd
		iwf = k_fullpackageFolder + ":" + k_pvfolder
		NVAR/SDFR=$iwf xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog, ihow, ionstack, bonstack
		SVAR/SDFR=$iwf ieqn
		xyhow = scv.xyhow
		mpp = scv.mpp
		xyiwidth = scv.xyiwidth
		xyiheight = scv.xyiheight
		ihow = scv.ihow
		ionstack = scv.ionstack
		ieqn = scv.ieqn
		bonstack = scv.bonstack
	endif
	
	return 0
end

// function to fix variable names
// lost translations in update from 1.x --> 2.x
// need: xyhow, pivalue, pilog, mpp, xyiwidth, xyiheight
// may have: xyhow, xympixel, xylmpixel, xyiwidth, xyliwidth
Function Sf_FixCalibrateVariables(wf)
	string wf
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wf
	if (exists("xympixel") == 2)
		NVAR xympixel, xylmpixel, xyiwidth, xyliwidth
		Variable/G mpp
		xyiwidth *= 10^xyliwidth
		Rename xympixel pivalue
		Rename xylmpixel pilog
		Rename xyliwidth xyiheight
		Variable/G xyiheight
	 	xyiheight = xyiwidth
	endif
	SetDataFolder cdf
	return 0
end

// get calibration variables
Function Sf_RecallCalibrateVariables(scv)
	STRUCT S_CalibrateVariables &scv

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string iwf
	
	// check local
	iwf = "root:" + cwdfolder + ":" + k_pvfolder
	if (DataFolderExists(iwf))
		// fix for update that did not correctly change parameter names
		Sf_FixCalibrateVariables(iwf)		
		NVAR/SDFR=$iwf/Z xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog, ihow, ionstack, bonstack
		SVAR/SDFR=$iwf ieqn
		scv.xyhow = xyhow
		scv.mpp = mpp
		scv.xyiwidth = xyiwidth
		scv.xyiheight = xyiheight
		scv.pivalue = pivalue
		scv.pilog = pilog
		scv.ihow = ihow
		scv.ionstack = ionstack
		scv.ieqn = ieqn
		scv.bonstack = bonstack
		return 0
	endif
	
	// check global
	if (DataFolderExists(k_fullpvfolder))
		iwf = k_fullpvfolder
		Sf_FixCalibrateVariables(iwf)		
		NVAR/SDFR=$iwf/Z xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog, ihow, ionstack, bonstack
		SVAR/SDFR=$iwf ieqn
		scv.xyhow = xyhow
		scv.mpp = mpp
		scv.xyiwidth = xyiwidth
		scv.xyiheight = xyiheight
		scv.pivalue = pivalue
		scv.pilog = pilog
		scv.ihow = ihow
		scv.ionstack = ionstack
		scv.ieqn = ieqn
		scv.bonstack = bonstack
		return 1
	endif
	
	// neither local or global
	return -1
end

// ***
// HELPER FUNCTIONS

// draw tab1 - calibrate
Function disp_Tab1Draw(pwidth,pheight, ptabpos)
	variable pwidth, pheight, ptabpos

	variable tvoffset = ptabpos + 30
	variable tleft = 20
	
	TabControl tab_main, value=1, userdata(ptab)="1"
	
	TitleBox calxy_tab1,pos={tleft-8,tvoffset},size={84,22},title="XY Scale"
	TitleBox calxy_tab1,fSize=16,frame=0,fStyle=1,disable=1
	
	Button buttonclearlocalscale_tab1,pos={pwidth-280,tvoffset},size={76,16},title="Clear Local"
	Button buttonclearlocalscale_tab1,proc=X_ButtonProc,disable=1
	
	Button buttonclearglobalscale_tab1,pos={pwidth-180,tvoffset},size={76,16},title="Clear Global"
	Button buttonclearglobalscale_tab1,proc=X_ButtonProc,disable=1
	
	CheckBox checkxyglobal_tab1,pos={pwidth-95,tvoffset+2},size={76,16},title="save as global",value=0, disable=1

	tvoffset += 25
	
	PopupMenu popupmethodxy_tab1,pos={20,tvoffset},size={113,23},title="How"
	PopupMenu popupmethodxy_tab1,mode=1,popvalue="pixel size",value= #"\"pixel size;image width;image height;\""
	PopupMenu popupmethodxy_tab1,disable=1,proc=X_PopMenuProc
	
	SetVariable psizem_tab1, pos={12+150,tvoffset},size={85,16},title="[m]",format="%1.3f"
	SetVariable psizem_tab1, value=_NUM:1, limits={0.1,10,0.1}, disable=1
	SetVariable psizelm_tab1, pos={12+240,tvoffset-3},size={70,16},title="\[010\Sn\M"
	SetVariable psizelm_tab1, value=_NUM:1, limits={-10,10,1}, disable=1
	
	Button buttonSetXY_tab1,pos={pwidth-85,tvoffset-5},size={75,25},proc=X_ButtonProc,title="Apply", disable=1

	// calibrate intensity
	
	tvoffset += 28
	
	TitleBox rescale_tab1,pos={tleft-8,tvoffset},size={84,22},title="Intensity"
	TitleBox rescale_tab1,fSize=16,frame=0,fStyle=1,disable=1
	
	PopupMenu popupmethodcalI_tab1,pos={tleft+80,tvoffset},size={113,23},title="How"
	PopupMenu popupmethodcalI_tab1,mode=1,popvalue="pixel size",value= #"\"Invert;Normalize Height;Normalize Flux Density;Equation;\""
	PopupMenu popupmethodcalI_tab1,disable=1,proc=X_PopMenuProc
	
	SetVariable setvarcalIEQN_tab1, pos={tleft+200,tvoffset+2},size={180,25},value=_STR:"",disable=1

	tvoffset += 25
			
	PopupMenu popupIonStack_tab1,pos={pwidth-185,tvoffset},size={100,23},title=""
	PopupMenu popupIonStack_tab1,mode=1,value="This Layer;Each Layer;All Layers;",disable=1

	Button buttonSetcalI_tab1,pos={pwidth-85,tvoffset-5},size={75,25},proc=X_ButtonProc,title="Apply", disable=1
	
	// remove background
	
	tvoffset += 28
		
	TitleBox background_tab1,pos={tleft-8,tvoffset},size={84,22},title="Background"
	TitleBox background_tab1,fSize=16,frame=0,fStyle=1,disable=1	

	PopupMenu popupBckgHow_tab1,pos={tleft+100,tvoffset},size={25,25},title=""
	PopupMenu popupBckgHow_tab1,mode=1,value="-;Grid;Grid X;Grid R;Grid T;",disable=1,proc=X_PopMenuProc

	Button buttonsetbckgrnd_tab1,pos={tleft+168,tvoffset},size={40,20},proc=X_ButtonProc,title="Set"

	// manual points

	Button buttonsetbckgrnd_tab1,userdata="start",disable=1
	Button buttoneditbckpoints_tab1,pos={tleft+216,tvoffset},size={40,20},proc=X_ButtonProc,title="Edit",disable=1
	Button buttonclearbckpoints_tab1,pos={tleft+264,tvoffset},size={40,20},proc=X_ButtonProc,title="Clear",disable=1
	Button buttonrestorebckpoints_tab1,pos={tleft+314,tvoffset},size={60,20},proc=X_ButtonProc,title="Restore",disable=1
	
	// grid points
	
	SetVariable setvarbckgx_tab1,pos={tleft+216,tvoffset+2},size={50,25},title="x",limits={4,40,1},value=_NUM:4, disable=1
	SetVariable setvarbckgy_tab1,pos={tleft+274,tvoffset+2},size={50,25},title="y",limits={4,40,1},value=_NUM:4, disable=1
	SetVariable setvarbckgmargin_tab1,pos={tleft+326,tvoffset+2},size={60,25},title="m",limits={0.01,1,0.01},value=_NUM:0.01, disable=1
	
	tvoffset += 28
	
	SetVariable setvarplorder_tab1,pos={tleft,tvoffset+2},size={80,25},title="Order"
	SetVariable setvarplorder_tab1,limits={1,4,1},value=_NUM:1, disable=1

	PopupMenu popupBckNegs_tab1,pos={tleft+90,tvoffset},size={25,25},title="(-)?"
	PopupMenu popupBckNegs_tab1,mode=1,value="Offset;Allow;Zero Out;",disable=1
	
	PopupMenu popupBckonStack_tab1,pos={pwidth-185,tvoffset},size={100,23},title=""
	PopupMenu popupBckonStack_tab1,mode=1,value="This Layer;Each Layer;All Layers;",disable=1

	Button buttonRemoveBckg_tab1,pos={pwidth-85,tvoffset-5},size={75,25},proc=X_ButtonProc,title="Remove"
	Button buttonRemoveBckg_tab1,disable=1
 
	return 0
end

// update tab 1
Function disp_updateTab1()

	variable istack = f_IsStack()
	variable hasBGM = f_HasBckgMrkrs()
	variable hasGBGM = f_HasGlobalBckgMrkrs()
	variable hasStoredBckgrnd = f_HasStoredBckgrnd()

	SetVariable setvarSize_tabM, win=$k_fullpanel, value=_STR:f_ImageSizeText()

	// ** Calibrate XY

	Button buttonclearlocalscale_tab1, win=$k_fullpanel, disable=(f_HasXYScale(1)==0)			
	Button buttonclearglobalscale_tab1, win=$k_fullpanel, disable=(f_HasXYScale(0)==0)			

	STRUCT S_CalibrateVariables scv
	Sf_RecallCalibrateVariables(scv)
	if (scv.xyhow!=0)
		PopupMenu popupmethodxy_tab1, win=$k_fullpanel, mode=scv.xyhow
		SetVariable psizem_tab1, win=$k_fullpanel, value=_NUM:scv.pivalue
		SetVariable psizelm_tab1, win=$k_fullpanel, value=_NUM:scv.pilog
	else
		SetVariable psizem_tab1, win=$k_fullpanel, value=_NUM:1
		SetVariable psizelm_tab1, win=$k_fullpanel, value=_NUM:0
	endif

	// ** Calibrate Intensity
	
	Sf_GetCalibrationInputs(scv)
	SetVariable setvarcalIEQN_tab1, win=$k_fullpanel, disable=(scv.ihow != 4)
	PopupMenu popupiOnStack_tab1, win=$k_fullpanel, disable=(!istack)
	
	// ** Remove Background

	ControlInfo/W=$k_fullpanel popupBckgHow_tab1
	T1_SetBckgrndHow(v_value)

	SetVariable setvarplorder_tab1, win=$k_fullpanel, disable=0
	Button buttonRemoveBckg_tab1, win=$k_fullpanel, disable=(2*!hasBGM)
	PopupMenu popupBckOnStack_tab1, win=$k_fullpanel, disable=(!istack)

	return 0
end

Function T1_SetBckgrndHow(how)
	variable how

	variable hasBGM = f_HasBckgMrkrs()
	variable hasGBGM = f_HasGlobalBckgMrkrs()
	variable hasStoredBckgrnd = f_HasStoredBckgrnd()
	
	switch(how)
		case 1:	// manual
			Button buttonsetbckgrnd_tab1,win=$k_fullpanel,disable=0
			Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=(2*!hasBGM)
			Button buttonclearbckpoints_tab1, win=$k_fullpanel,disable=(2*!hasBGM)
			Button buttonrestorebckpoints_tab1 win=$k_fullpanel,disable=(2*!hasStoredBckgrnd)
	
			SetVariable setvarbckgx_tab1, win=$k_fullpanel, disable=1
			SetVariable setvarbckgy_tab1, win=$k_fullpanel, disable=1
			SetVariable setvarbckgmargin_tab1,win=$k_fullpanel, disable=1
			break			
		default:	// grid
			Button buttonsetbckgrnd_tab1,win=$k_fullpanel,disable=0
			Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=1
			Button buttonclearbckpoints_tab1, win=$k_fullpanel,disable=1
			Button buttonrestorebckpoints_tab1 win=$k_fullpanel,disable=1

			SetVariable setvarbckgx_tab1, win=$k_fullpanel, disable=0
			SetVariable setvarbckgy_tab1, win=$k_fullpanel, disable=0
			SetVariable setvarbckgmargin_tab1,win=$k_fullpanel, disable=0
			break
	endswitch
	
	return 0
end

// ***
// ACTIONS

// clear XY scale
Function T1_ClearXYScale(where)
	variable where
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string iwf
	switch(where)
		case 0:		// global
			iwf = k_fullpackagefolder + ":" + k_pvfolder
			break
		case 1:		// local
			iwf = "root:" + cwdfolder + ":" + k_pvfolder
			break
	endswitch
	
	KillDataFolder/Z $iwf

	return 0
end

// set xy method
Function T1_SetXYMethod(how)
	variable how
	
//	PopupMenu mrulera_tab1, win=$k_fullpanel, disable = 1
	switch(how)
		case 4:	// ruler length
			PopupMenu mrulera_tab1, win=$k_fullpanel, disable = 0
			break
	endswitch
	return 0
end

// set the xy calibration
Function T1_SetXY()
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string swf = "root:" + cwdfolder
	wave/SDFR=$swf cwave = $cwfile
	
	variable iwpx, iw, asglobal
	variable pwidth = DimSize(cwave,0)
	variable pheight = DimSize(cwave,1)

	STRUCT S_CalibrateVariables scv

	// get values from panel
	
	Sf_GetCalibrationInputs(scv)
	
	switch(scv.xyhow)
		case 1:	// pixel size
			scv.mpp = scv.pivalue*10^scv.pilog
			scv.xyiwidth = pwidth*scv.mpp
			scv.xyiheight = pheight*scv.mpp
			break
		case 2:	// image width
			scv.xyiwidth = scv.pivalue*10^scv.pilog
			scv.mpp = scv.xyiwidth/pwidth
			scv.xyiheight = pheight*scv.mpp
			break
		case 3:	// image height
			scv.xyiheight = scv.pivalue*10^scv.pilog
			scv.mpp = scv.xyiheight/pheight
			scv.xyiwidth = pwidth*scv.mpp
			break
	endswitch
	
	// save values
	Sf_SaveCalibrateVariables(scv)
	
	return 0
end

// set the location points for the background
Function T1_SetBackgroundPoints(how)
	variable how
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string usD, rcwd, rcwdf
	variable whi, hasBGM = f_HasBckgMrkrs()
	variable nx, ny, margin, xc, xr, yc, yr
	STRUCT S_MarqueeCoordinates smc
	
	usD = GetUserData(k_fullPanel,"buttonsetbckgrnd_tab1","")
	strswitch(usD)
		case "start":
			switch(how)
				case 1:		// manual
					Button buttonsetbckgrnd_tab1,win=$k_fullpanel,userdata="end",title="End..."
					SetWindow $k_imgDisplay, userdata(draw)="active"
					SetDrawLayer/W=$k_imgDisplay ProgFront
					GraphWaveDraw/W=$k_imgDisplay/L/T/O imgT_bckgY,imgT_bckgX
					ModifyGraph/W=$k_imgDisplay mode=3,marker=8,msize=5,mrkThick=2
					break
				case 2:		// grid
					ControlInfo/W=$k_fullpanel setvarbckgx_tab1
					nx = v_value
					ControlInfo/W=$k_fullpanel setvarbckgy_tab1
					ny = v_value
					ControlInfo/W=$k_fullpanel setvarbckgmargin_tab1
					margin = v_value
					make_GridPoints(nx,ny,margin=margin)
					break
				case 3:		// grid boX
					ControlInfo/W=$k_fullpanel setvarbckgx_tab1
					nx = v_value
					ControlInfo/W=$k_fullpanel setvarbckgy_tab1
					ny = v_value
					ControlInfo/W=$k_fullpanel setvarbckgmargin_tab1
					margin = v_value
					if (f_HasMQ())
						Sf_GetMQCoordinates(smc)
						xc = (smc.pgright + smc.pgleft)/2
						yc = (smc.pgbottom + smc.pgtop)/2
						xr = (smc.pgright - smc.pgleft)/2
						yr = (smc.pgbottom - smc.pgtop)/2
						make_GridPoints(nx,ny,mode=1,margin=margin,xcenter=xc,ycenter=yc,xr=xr,yr=yr)
					else
						make_GridPoints(nx,ny,margin=margin)
					endif
					break
				case 4:		// grid Radial
					ControlInfo/W=$k_fullpanel setvarbckgx_tab1
					nx = v_value
					ControlInfo/W=$k_fullpanel setvarbckgy_tab1
					ny = v_value
					ControlInfo/W=$k_fullpanel setvarbckgmargin_tab1
					margin = v_value
					if (f_HasMQ())
						Sf_GetMQCoordinates(smc)
						xc = (smc.pgright + smc.pgleft)/2
						yc = (smc.pgbottom + smc.pgtop)/2
						xr = (smc.pgright - smc.pgleft)/2
						yr = (smc.pgbottom - smc.pgtop)/2
						make_GridPoints(nx,ny,mode=2,margin=margin,xcenter=xc,ycenter=yc,xr=xr,yr=yr)
					else
						make_GridPoints(nx,ny,margin=margin)
					endif
					break
				case 5:		// grid Threshold
					ControlInfo/W=$k_fullpanel setvarbckgx_tab1
					nx = v_value
					ControlInfo/W=$k_fullpanel setvarbckgy_tab1
					ny = v_value
					ControlInfo/W=$k_fullpanel setvarbckgmargin_tab1
					margin = v_value
					if (f_ThresholdExists())
						make_GridPoints(nx,ny,mode=3,margin=margin,xcenter=xc,ycenter=yc,xr=xr,yr=yr)
					else
						make_GridPoints(nx,ny,margin=margin)
					endif
					break
			endswitch
			break
		case "end":
			DoUpdate/W=$k_imgDisplay
			Button buttonsetbckgrnd_tab1,win=$k_fullpanel,userdata="start",title="Set"
			Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=0
			Button buttonclearbckpoints_tab1,win=$k_fullpanel, disable=0
			Button buttonRemoveBckg_tab1, win=$k_fullpanel, disable=(2*!hasBGM)
			GraphWaveEdit/W=$k_imgDisplay/T=1 scalewy
			GraphNormal/W=$k_imgDisplay
			SetDrawLayer/W=$k_imgDisplay UserFront
			SetWindow $k_imgDisplay, userdata(draw)=""
			// put the data points in the working folder
			DFREF cdf = GetDataFolderDFR()	
			wave/SDFR=cdf cbx = imgT_bckgX
			wave/SDFR=cdf cby = imgT_bckgY
			
			rcwd = "root:" + cwdfolder
			SetDataFolder $rcwd
			rcwd += ":imgT_background"
			NewDataFolder/O $rcwd
			SetDataFolder cdf
						
			rcwdf = rcwd + ":imgT_bckgY"
			duplicate/O cby $rcwdf
			rcwdf = rcwd + ":imgT_bckgX"			
			duplicate/O cbx $rcwdf
			
			break
	endswitch

	return 0
end

// clear the location points for the background
Function T1_EditBackgroundPoints()
	
	//wave/SDFR=root imgT_bckgY, imgT_bckgX
	
	SetDrawLayer/W=$k_imgDisplay ProgFront
	GraphWaveEdit/W=$k_imgDisplay/T=1 imgT_bckgY
	Button buttonsetbckgrnd_tab1,win=$k_fullpanel,userdata="end",title="End..."
	Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=2
	Button buttonclearbckpoints_tab1,win=$k_fullpanel, disable=2
	Button buttonRemoveBckg_tab1, win=$k_fullpanel, disable=2
	SetWindow $k_imgDisplay, userdata(draw)="active"
	
	return 0
end

// clear the location points for the background
Function T1_ClearBackgroundPoints()
		
	RemoveFromGraph/W=$k_imgDisplay/Z imgT_bckgY
	killwaves/Z imgT_bckgY, imgT_bckgX
	Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=2
	Button buttonclearbckpoints_tab1,win=$k_fullpanel, disable=2
	
	return 0
end

// remove background
Function T1_RemoveBackground()
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string wdf = "root:" + cwdfolder
	string sfile, wfile
	variable cplane = f_PlaneNumber()
	variable sf, ic, vmin //, vmax //, fpmax = 2^64 - 1
	variable bckgexists=0, nimgs, imin=0, po, onplane, negatives
	
	// avoid background removal on a background
	// set existence of background
	if (f_isBackground())
		bckgexists = 1
		sfile = ReplaceString("_bg",cwfile,"")
		wave/SDFR=$wdf simg = $sfile
		wfile = sfile + "_bg"
	else
		wave/SDFR=$wdf simg = $cwfile
		wfile = cwfile + "_bg"
		sfile = cwfile
	endif

	STRUCT S_BackgroundParams sbg
	Sf_GetBackgroundParams(sbg)
	
	po = sbg.plorder
	onplane = sbg.onplane
	negatives = sbg.negatives

	// set up for background removal
	wave/SDFR=$wdf/Z bimg = $wfile
	bckgexists = waveexists(bimg)
	
	// remove the background
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdf
	
	// create new background
	if (bckgexists==0)
		MatrixOP/O $wfile = fp64(simg)
		wave wbimg = $wfile
		wbimg = 0
	else
		// assure that we are at 64 bit
		if (f_ImageBitDepth()!=64)
			wave wbimg = $wfile
			Redimension/D wbimg
		endif
	endif
	
	// extract wave to create background
	if (f_isStack())
		nimgs = f_NImagesInStack()
	else
		nimgs = 1
		onplane = 0
	endif
	
	// create the RoI mask
	wave RoIMask = GenerateBckgMask(simg)
	
	// wave declarations
	wave/SDFR=$wdf wbimg = $wfile

	// remove background
	switch(onplane)
		case 0:		// not a stack
			MatrixOP/O fsbimg = fp64(simg)
			ImageRemoveBackground/P=(po)/O/R=RoIMask fsbimg
			switch(negatives)
				case 1:	// offset
					vmin = wavemin(fsbimg)
					MatrixOP/O fbimg = (fsbimg - vmin)
					break
				case 2:	// allow
					MatrixOP/O fbimg = fsbimg
					break
				case 3:	// zero out
					MatrixOP/O fbimg = fsbimg
					fbimg = fsbimg < 0 ? 0 : fsbimg
					break
			endswitch
			wbimg = fbimg
			break
		case 1:		// this plane only
			MatrixOP/O fsbimg = fp64(layer(simg,cplane))
			ImageRemoveBackground/O/P=(po)/R=RoIMask fsbimg
			switch(negatives)
				case 1:	// offset
					vmin = wavemin(fsbimg)
					MatrixOP/O fbimg = (fsbimg - vmin)
					break
				case 2:	// allow
					MatrixOP/O fbimg = fsbimg
					break
				case 3:	// zero out
					MatrixOP/O fbimg = fsbimg
					fbimg = fsbimg < 0 ? 0 : fsbimg
					break
			endswitch
			ImageTransform/D=fbimg/P=(cplane) setPlane, wbimg
			break
		case 2:		// all planes
			for (ic=0;ic<nimgs;ic+=1)
				MatrixOP/O fsbimg = fp64(layer(simg,ic))
				ImageRemoveBackground/O/P=(po)/R=RoIMask fsbimg
				switch(negatives)
					case 1:	// offset
						vmin = wavemin(fsbimg)
						MatrixOP/O fbimg = (fsbimg - vmin)
						break
					case 2:	// allow
						MatrixOP/O fbimg = fsbimg
						break
					case 3:	// zero out
						MatrixOP/O fbimg = fsbimg
						fbimg = fsbimg < 0 ? 0 : fsbimg
						break
				endswitch
				ImageTransform/D=fbimg/P=(ic) setPlane, wbimg
			endfor
			break
	endswitch
	
	killwaves/Z fbimg, fsbimg, RoIMask
	CopyScales simg wbimg
	
	SetDataFolder cdf

	make/N=3/T/FREE ptxt={"Plane","Plane Order","On Plane Only"}
	make/N=3/D/FREE pvals={cplane,po,onplane}
	ImgT_PrintReport("Removed Background",ptxt,pvals,image = cwdfolder + " / " + cwfile)
	
	return 0
end

ThreadSafe Function TS_RemoveBackground(WAVE fbimgSTACK, WAVE RoIMask, variable po, variable ic)

    variable sf
    MatrixOP/FREE fbimg = layer(fbimgSTACK,ic)
    ImageRemoveBackground/O/W/P=(po)/R=RoIMask fbimg
    WaveStats/Q/M=1 fbimg
    sf = 255/(V_max - V_min)
    MatrixOP/O fbimg = fp64(sf*(fbimg - V_min))
    fbimgSTACK[][][ic] = fbimg[p][q]

    return 0
end

// generate an RoI mask from wimg
// how: 1 mask inside; 2 mask outside
// which: 0 histogram, 1 background, 2 threshold
Function/WAVE GenerateBckgMask(wimg)
	wave wimg
	
	variable npts, ic

	duplicate/O wimg mwimg
	wave imgMask = mwimg
	if (WaveDims(wimg) == 3)
		Redimension/B/U/N=(-1,-1,1) imgMask
	else
		Redimension/B/U imgMask
	endif
	
	wave/SDFR=root: imgT_bckgY, imgT_bckgX
	npts = numpnts(imgT_bckgY)
	imgMask = 64
	for (ic=0;ic<npts;ic+=1)
		imgMask[x2pnt(imgMask,imgT_bckgX[ic])][x2pnt(imgMask,imgT_bckgY[ic])] = 1
	endfor
	
	return mwimg
end

Function T1_ShowBackground()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwfile
	
	cwfile += "_bg"
	
	disp_ShowAnImage()

	return 0
end

// normalize the input to height
Static Function norm_byheight(wave theImg)

	variable scf
	
	WaveStats/Q theImg
	scf = 1/(V_max - V_min)
	MatrixOP/FREE theNImg = scf*(theImg - V_min)
	duplicate/O theNImg theImg
	
	return scf
end

// normalize the input to flux density
Static Function norm_byfluxdensity(wave theImg, variable isize)

	variable scf
	
	WaveStats/Q theImg
	scf = isize/V_sum
	MatrixOP/FREE theNImg = scf*theImg
	duplicate/O theNImg theImg
	
	return scf
end

// calibrate image intensity
Function T1_CalibrateIntensity() //(variable how)

	wave simg = f_imgwaveDisplayed()
	variable scf, iSize, bd = f_imageBitDepth()
	variable nplanes, npl, ic
	variable isStack = f_isStack()
	variable scfmax = 2^bd - 1
	string wdfldr = f_wdfstr(0)
	string imgstr = f_wdfstr(1)
	string nimgstr
	string calcmd

	STRUCT S_CalibrateVariables scv
	Sf_GetCalibrationInputs(scv)
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdfldr
	
	switch(scv.ihow)
		case 1:	// invert
			if (isStack && (scv.ionstack == 1))
				ic = f_PlaneNumber()
				MatrixOP/FREE fplane = layer(simg,ic))
				ImageTransform/O invert fplane
				CopyScales simg fplane
				ImageTransform/P=(ic)/D=fplane setPlane simg
			else
				ImageTransform/O invert simg
			endif
			break

		case 2:	// normalize by height

			nimgstr = imgstr + "_nbh"

			if (!isStack)
				MatrixOP/O theImg = fp64(simg)
				norm_byheight(theImg)
				CopyScales simg theImg
				duplicate/O theImg $nimgstr
				wave nbhimg = $nimgstr
			else
				// create new _nbh as needed
				wave/Z nbhimg = $nimgstr
				if (!WaveExists(nbhimg))
					MatrixOP/O $nimgstr = fp64(simg)
					wave nbhimg = $nimgstr
					CopyScales simg nbhimg
					nbhimg = 0
				endif
				wave nbhimg = $nimgstr
				nplanes = f_NImagesInStack()
				switch(scv.ionstack)
					case 1:	// this layer
						npl = f_PlaneNumber()
						MatrixOP/O theImg = fp64(layer(simg,npl))
						norm_byheight(theImg)
						CopyScales simg theImg
						ImageTransform/P=(npl)/D=theImg setPlane nbhimg					
						break
					case 2:	// each layer
						MatrixOP/O theImg = fp64(simg)
						for(ic=0;ic<nplanes;ic+=1)
							MatrixOP/O thePlane = layer(theImg,ic)
							norm_byheight(thePlane)
							ImageTransform/P=(ic)/D=thePlane setPlane theImg
						endfor
						CopyScales simg theImg
						duplicate/O theImg nbhimg					
						break
					case 3:	// all layers
						MatrixOP/O theImg = fp64(simg)
						norm_byheight(theImg)
						CopyScales simg theImg
						duplicate/O theImg nbhimg					
						break
				endswitch
				killwaves/Z theImg, thePlane
			endif
			
			break

		case 3:	// normalize by flux density (total intensity)
		
			nimgstr = imgstr + "_nbf"
			iSize = DimSize(simg,0)*DimSize(simg,1)

			if (!isStack)
				MatrixOP/O theImg = fp64(simg)
				norm_byfluxdensity(theImg, isize)
				CopyScales simg theImg
				duplicate/O theImg $nimgstr
				wave nbfimg = $nimgstr
			else
				// create new _nbf as needed
				wave/Z nbfimg = $nimgstr
				if (!WaveExists(nbfimg))
					MatrixOP $nimgstr = fp64(simg)
					wave nbfimg = $nimgstr
					CopyScales simg nbfimg
					nbfimg = 0
				endif
				wave nbfimg = $nimgstr
				nplanes = f_NImagesInStack()
				
				switch(scv.ionstack)
					case 1:	// this layer
						npl = f_PlaneNumber()
						MatrixOP/O theImg = fp64(layer(simg,npl))
						norm_byfluxdensity(theImg, isize)
						CopyScales simg theImg
						ImageTransform/P=(npl)/D=theImg setPlane nbfimg						
						break
					case 2:	// each layer
						MatrixOP/O theImg = fp64(simg)
						for(ic=0;ic<nplanes;ic+=1)
							MatrixOP/O thePlane = layer(theImg,ic)
							norm_byfluxdensity(thePlane, isize)
							ImageTransform/P=(ic)/D=thePlane setPlane theImg
						endfor
						CopyScales simg theImg
						duplicate/O theImg nbfimg					
						break
					case 3:	// all layers
						MatrixOP/O theImg = fp64(simg)/nplanes
						norm_byfluxdensity(theImg, isize)
						CopyScales simg theImg
						duplicate/O theImg nbfimg	
						break
				endswitch
				killwaves/Z theImg, thePlane
			endif

			break

		case 4:	// use an equation
		
			nimgstr = imgstr + "_eqn"
			calcmd = T1_getcalibrateIeqn(scv.ieqn)
			
			if (!isStack)
				MatrixOP/O tmpsrc = fp64(simg)
				duplicate tmpsrc tmpstck
				Execute/Q/Z calcmd
				CopyScales simg tmpstck
				duplicate/O tmpstck $nimgstr
				wave neqnimg = $nimgstr
			else
				// create new _neqn as needed
				wave/Z neqnimg = $nimgstr
				if (!WaveExists(neqnimg))
					MatrixOP $nimgstr = fp64(simg)
					wave neqnimg = $nimgstr
					CopyScales simg neqnimg
					neqnimg = 0
				endif
				wave neqnimg = $nimgstr
				nplanes = f_NImagesInStack()
				
				switch(scv.ionstack)
					case 1:	// this layer
						npl = f_PlaneNumber()
						MatrixOP/O tmpsrc = fp64(layer(simg,npl))
						duplicate/O tmpsrc tmpstck
						Execute/Q/Z calcmd
						CopyScales simg tmpstck
						ImageTransform/P=(npl)/D=tmpstck setPlane neqnimg						
						break
					case 2:	// each layer
						MatrixOP/O theImgStck = fp64(simg)
						for(ic=0;ic<nplanes;ic+=1)
							MatrixOP/O tmpsrc = layer(theImgStck,ic)
							duplicate/O tmpsrc, tmpstck
							Execute/Q/Z calcmd
							ImageTransform/P=(ic)/D=tmpstck setPlane tmpsrc
						endfor
						CopyScales simg tmpsrc
						duplicate/O tmpsrc neqnimg		
						break
					case 3:	// all layers
						MatrixOP/O tmpsrc = fp64(simg)
						duplicate/O tmpsrc, tmpstck
						Execute/Q/Z calcmd
						duplicate/O tmpstck neqnimg					
						break
				endswitch
				killwaves/Z tmpsrc, tmpstck, theImgStck
			endif

			break
	endswitch
	
	SetDataFolder cdf
	
	return 0
end

Static Function/S T1_getcalibrateIeqn(string strEQN)

	string rtstr = "", pstr
	
	rtstr = ReplaceString("I",strEQN,"tmpsrc")
	rtstr = ReplaceString("x",rtstr,"(pnt2x(tmpsrc,p))")
	rtstr = ReplaceString("y",rtstr,"(pnt2x(tmpsrc,q))")
	sprintf rtstr, "tmpstck = %s", rtstr
	
	return rtstr

end

// restore background points
Function T1_RestoreBackgroundPoints([disponly])
	variable disponly

	DFREF pf = $k_fullpackagefolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	if (ParamIsDefault(disponly))
		string bckfolder = "root:" + cwdfolder + ":imgT_background"
		wave/SDFR=$bckfolder iX = imgT_bckgX, iY = imgT_bckgY
		duplicate/O iY imgT_bckgY
		duplicate/O iX imgT_bckgX
	endif
	RemoveFromGraph/Z/W=$k_imgDisplay imgT_bckgY
	AppendtoGraph/W=$k_imgDisplay/T imgT_bckgY vs imgT_bckgX
	ModifyGraph/W=$k_imgDisplay mode=3,marker=8,msize=5,mrkThick=2
	DoUpdate/W=$k_imgDisplay
	GraphWaveEdit/W=$k_imgDisplay/T=1 scalewy
	GraphNormal/W=$k_imgDisplay
	SetWindow $k_imgDisplay, userdata(draw)=""
	
	return 0

end

// make a set of grid points
Static Function make_GridPoints(numh,numv,[mode,margin,hmargin,vmargin,xcenter,ycenter,xr,yr])
	variable numh, numv, mode, margin, hmargin, vmargin, xcenter, ycenter, xr, yr

	variable xc, yc, ic, iwidth, iheight, px, py, ox, oy
	variable dx, dy, xo, yo, npnts
	variable rc, rr, excp = 0
	string strthimg
	
	numh = numh < 4 ? 4 : numh
	numv = numv < 4 ? 4 : numv
	
	if (ParamIsDefault(mode))
		mode = 1
	endif
	if (ParamIsDefault(margin))
		margin = 0.02
	else
		margin += 0.02
	endif
	if (ParamIsDefault(hmargin))
		hmargin = margin
	else
		hmargin += 0.02
	endif
	if (ParamIsDefault(vmargin))
		vmargin = margin
	else
		vmargin += 0.02
	endif
	
	iwidth = f_ImageSize()[1]
	iheight = f_ImageSize()[2]
	
	xo = hmargin*iwidth
	yo = vmargin*iheight
	
	dx = (1 - 2*hmargin)*iwidth/(numh-1)
	dy = (1 - 2*vmargin)*iheight/(numv-1)
	
	npnts = numh*numv
	
	if (!ParamIsDefault(xr))
		if (ParamIsDefault(xcenter))
			xcenter = iwidth/2
		endif
		excp = 1
	endif
	if (!ParamIsDefault(yr))
		if (ParamIsDefault(ycenter))
			ycenter = iheight/2
		endif
		excp = 1
	endif
	
	DFREF cdf=GetDataFolderDFR()
	
	SetDataFolder root:
	
	make/N=(npnts)/D/O imgT_bckgX, imgT_bckgY
	ic = 0
	rc = sqrt((xr+0)^2 + (yr+0)^2)
	for (xc=0;xc<numh;xc+=1)
		for (yc=0;yc<numv;yc+=1)
			px = xo + xc*dx
			py = yo + yc*dy
			if (excp == 1)
				switch(mode)
					case 1:	// box mode
						ox = (px < (xcenter - xr) || px > (xcenter + xr)) ? 1 : 0
						oy = (py < (ycenter - yr) || py > (ycenter + yr)) ? 1 : 0
						px = (ox || oy) ? px : NaN
						py = (ox || oy) ? py : NaN
						break
					case 2:	// radial mode
						rr = sqrt((px-xcenter)^2 + (py-ycenter)^2)
						px = rr < rc ? NaN : px
						py = rr < rc ? NaN : py
						break
					case 3:	// threshold mode
						strthimg = f_DisplayImageID(3)
						if (!f_IsThreshold())
							strthimg += "_th"
						endif
						wave/SDFR=$(f_DisplayImageID(1)) thimg = $strthimg
						npnts = f_PlaneNumber()
						MatrixOP/FREE timg = layer(thimg,npnts)
						ox = timg[px][py] > 0 ? 1 : 0
						oy = timg[px][py] > 0 ? 1 : 0
						px = (ox || oy) ? NaN : px
						py = (ox || oy) ? NaN : py
						break
				endswitch
			endif
			imgT_bckgX[ic] = px
			imgT_bckgY[ic] = py
			ic += 1
		endfor
	endfor
	
	if (excp == 1)
		WaveTransform zapNaNs imgT_bckgX
		WaveTransform zapNaNs imgT_bckgY
	endif
	
	T1_RestoreBackgroundPoints(disponly=1)

	SetDataFolder cdf
	
	return 0
		
end

Function T1_removeBckgrndPoint()

	string rf = "root:"
	wave/SDFR=$rf bx = imgT_bckgX, by = imgT_bckgY
	
	variable vx, vy, pp
	
	GetLastUserMenuInfo
	if (strlen(S_traceName) != 0)
		vx = AxisValFromPixel(S_graphname,"top",V_mouseX)
		vy = AxisValFromPixel(S_graphname,"left",V_mouseY-cbheight)
		
		duplicate/FREE bx bfx, bT
		duplicate/FREE by bfy
		
		bfx = ((bx < vx*(0.95)) || (bx > vx*(1.05))) ? 1 : 0
		bfy = ((by < vy*(0.95)) || (by > vy*(1.05))) ? 1 : 0
		bT = bfx|bfy
		
		FindValue/V=0 bT
		pp = v_value
		bx[pp] = NaN; by[pp] = NaN
		WaveTransform zapNaNs bx
		WaveTransform zapNaNs by
	endif
	
	return 0
end